import './routes/index.ts';
import { app } from './app.ts';
import type { App } from '@kevisual/router';
import { User } from './models/user.ts';
// import { addAuth } from '@kevisual/code-center-module/models';
// addAuth(app);
import { createCookie, getSomeInfoFromReq } from './routes/user/me.ts';

/**
 * 添加auth中间件, 用于验证token
 * 添加 id: auth 必须需要user成功
 * 添加 id: auth-can 可以不需要user成功，有则赋值
 *
 * @param app
 */
export const addAuth = (app: App) => {
  app
    .route({
      path: 'auth',
      id: 'auth',
    })
    .define(async (ctx) => {
      const token = ctx.query.token;
      if (!token) {
        app.throw(401, 'Token is required');
      }
      const user = await User.getOauthUser(token);
      if (!user) {
        app.throw(401, 'Token is invalid');
      }
      const someInfo = getSomeInfoFromReq(ctx);
      if (someInfo.isBrowser && !ctx.req?.cookies?.['token']) {
        createCookie({ accessToken: token }, ctx);
      }
      ctx.state.tokenUser = user;
    })
    .addTo(app);

  app
    .route({
      path: 'auth',
      key: 'can',
      id: 'auth-can',
    })
    .define(async (ctx) => {
      if (ctx.query?.token) {
        const token = ctx.query.token;
        const user = await User.getOauthUser(token);
        if (token) {
          ctx.state.tokenUser = user;
          const someInfo = getSomeInfoFromReq(ctx);
          if (someInfo.isBrowser && !ctx.req?.cookies?.['token']) {
            createCookie({ accessToken: token }, ctx);
          }
        } else {
          ctx.state.tokenUser = null;
        }
      }
    })
    .addTo(app);
};
addAuth(app);

app
  .route({
    path: 'auth',
    key: 'admin',
    id: 'auth-admin',
    isDebug: true,
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    if (!tokenUser) {
      ctx.throw(401, 'No User For authorized');
    }
    try {
      const user = await User.findOne({
        where: {
          id: tokenUser.id,
        },
      });
      if (!user) {
        ctx.throw(404, 'user not found');
      }
      user.setTokenUser(tokenUser);
      const orgs = await user.getOrgs();
      if (orgs.includes('admin')) {
        ctx.body = 'admin';
      } else {
        ctx.throw(403, 'forbidden');
      }
    } catch (e) {
      console.error(`auth-admin error`, e);
      console.error('tokenUser', tokenUser?.id, tokenUser?.username, tokenUser?.uid);
      ctx.throw(500, e.message);
    }
  })
  .addTo(app);
app
  .route({
    path: 'auth-check',
    key: 'admin',
    id: 'check-auth-admin',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    if (!tokenUser) {
      ctx.throw(401, 'No User For authorized');
    }

    try {
      const user = await User.findOne({
        where: {
          id: tokenUser.id,
        },
      });
      if (!user) {
        ctx.throw(404, 'user not found');
      }
      user.setTokenUser(tokenUser);
      const orgs = await user.getOrgs();
      if (orgs.includes('admin')) {
        ctx.body = 'admin';
        ctx.state.tokenAdmin = {
          id: user.id,
          username: user.username,
          orgs,
        };
        return;
      }
      ctx.body = 'not admin';
    } catch (e) {
      console.error(`auth-admin error`, e);
      console.error('tokenUser', tokenUser?.id, tokenUser?.username, tokenUser?.uid);
      ctx.throw(500, e.message);
    }
  })
  .addTo(app);

app
  .route({
    path: 'test',
    key: 'test',
  })
  .define(async (ctx) => {
    ctx.body = app.router.routes.map((item) => {
      return {
        path: item.path,
        key: item.key,
        description: item.description,
        validator: item.validator,
        // schema: item.schema,
      };
    });
  })
  .addTo(app);
